using System;
using System.Text;
using System.Configuration;

class Cs
{
public static void Main(String[] argv)
  {
  String [] args = Environment.GetCommandLineArgs();
  System.IO.FileStream f = new System.IO.FileStream(args[1], System.IO.FileMode.Open, System.IO.FileAccess.Read, System.IO.FileShare.Read, 8192);
  Yylex yy = new Yylex(f);
  Yytoken t;
  Console.Write("<pre><br>");

  while ((t = yy.yylex()) != null)
    Console.Write(t);

  Console.Write("</pre>");
  }
}

class Util
{
public static void IllChar(String s)
 {
 StringBuilder sb = new StringBuilder("Illegal character: <");
 for (int i = 0; i < s.Length; i++)
   if (s[i] >= 32)
     sb.Append(s[i]);
   else
     {
     sb.Append("^");
     sb.Append(Convert.ToChar(s[i]+'A'-1));
     }
 sb.Append(">");
 Console.Write(sb.ToString());
 }
} 

class Yytoken
  {
  public int m_index;
  public String m_text;
  public int m_line;
  public int m_charBegin;
  public int m_charEnd;

internal Yytoken(int index, String text, int line, int charBegin, int charEnd)
  {
  m_index = index;
  m_text = text;
  m_line = line;
  m_charBegin = charBegin;
  m_charEnd = charEnd;
  }

public override String ToString()
  {
  return "Token #"+ m_index + ": " + m_text
    + " (line "+ m_line + ")";
  }
}

//STRING=(\"([^\"]|\\\")*\")
%%
%line
%char
%state COMMENT
%state STRING_STATE

NONNEWLINE_WHITE_SPACE_CHAR=[\ \b\012]
TABULATION=[\t]
COMMENT_TEXT=([^/*$]|[^*$]"/"[^*$]|[^/$]"*"[^/$]|"*"[^/$]|"/"[^*$])*
COMMENT_DOUBLESLASH=//.*
NEW_LINE=((\r\n)|\n)
LIGNEVIDE=^\n((" "|\t)*\n)$
CHAR='[^']+'
NUMBER=[+-]?[0-9]+(\.[0-9]*)?

MC_PUBLIC     public
MC_PRIVATE    private
MC_PROTECTED  protected
MC_INTERNAL   internal

MC_NAMESPACE  namespace
MC_USING      using
MC_AS         as
MC_CLASS      class
MC_STATIC     static
MC_SEALED     sealed
MC_CONST      const
MC_OVERRIDE   override
MC_DELEGATE   delegate
MC_IMPLICIT   implicit
MC_EXPLICIT   explicit
MC_OPERATOR   operator
MC_TRY        try
MC_CATCH      catch
MC_FINALLY    finally
MC_OUT        out
MC_REF        ref
MC_PARAMS     params

MC_NEW        new
MC_RETURN     return
MC_THROW      throw
MC_NULL       null
MC_THIS       this
MC_BASE       base

MC_FOR        for
MC_FOREACH    foreach
MC_IN         in
MC_IF         if
MC_ELSE       else
MC_SWITCH     switch
MC_CASE       case
MC_DEFAULT    default
MC_WHILE      while
MC_BREAK      break
MC_GOTO       goto
MC_CONTINUE   continue

MC_STRING     string|String
MC_CHAR       char|Char
MC_INT        int
MC_DOUBLE     double
MC_FLOAT      float
MC_VOID       void
MC_BOOL       bool

IDF=[a-zA-Z_$][a-zA-Z_$0-9]*

MOTS_CLES {MC_PUBLIC}|{MC_CLASS}|{MC_FOR}|{MC_IF}|{MC_STATIC}|{MC_SWITCH}|{MC_WHILE}|{MC_SEALED}|{MC_PRIVATE}|{MC_STRING}|{MC_CHAR}|{MC_DEFAULT}|{MC_BREAK}|{MC_INT}|{MC_VOID}|{MC_CASE}|{MC_ELSE}|{MC_PROTECTED}|{MC_INTERNAL}|{MC_FOREACH}|{MC_IN}|{MC_DOUBLE}|{MC_FLOAT}|{MC_USING}|{MC_AS}|{MC_NAMESPACE}|{MC_CONST}|{MC_OVERRIDE}|{MC_NEW}|{MC_NULL}|{MC_RETURN}|{MC_DELEGATE}|{MC_THROW}|{MC_BOOL}|{MC_THIS}|{MC_BASE}|{MC_IMPLICIT}|{MC_EXPLICIT}|{MC_OPERATOR}|{MC_TRY}|{MC_CATCH}|{MC_FINALLY}|{MC_OUT}|{MC_REF}|{MC_PARAMS}|{MC_GOTO}|{MC_CONTINUE}
%% 

<YYINITIAL> {LIGNEVIDE} { /* this is newline */
Console.Write(ConfigurationSettings.AppSettings["return"]);
return null;
}

<YYINITIAL> {NEW_LINE} { /* this is newline */
Console.Write(ConfigurationSettings.AppSettings["return"]);
return null;
}

<YYINITIAL> {NONNEWLINE_WHITE_SPACE_CHAR} { /* this is whitespace */
  if (ConfigurationSettings.AppSettings["whitespace"] != null)
    Console.Write(ConfigurationSettings.AppSettings["whitespace"]);
  else
    Console.Write(yytext());
return null;
}

<YYINITIAL> {TABULATION} { /* this is whitespace */
Console.Write(ConfigurationSettings.AppSettings["tab"]);
return null;
}

<YYINITIAL> {COMMENT_DOUBLESLASH} {
  Console.Write (ConfigurationSettings.AppSettings["commentStart"] + yytext() + ConfigurationSettings.AppSettings["commentEnd"]);
  return null;
  }


<YYINITIAL> "/*" { /* comment begin (initial) */ yybegin(COMMENT);
Console.Write(ConfigurationSettings.AppSettings["commentStart"] + "/" + "*");
return null;
}

<COMMENT> {COMMENT_TEXT} {
  Console.Write(yytext());
 /* comment text here */
return null;
}

<YYINITIAL> "/*" {
  /* comment begin (initial) */
  yybegin(COMMENT);
  return null;
  }

<COMMENT> "/*" {
  /* comment begin (non-initial) */
  return null;
  }

<COMMENT> "*/" {
  /* comment end */
  Console.Write("*" + "/" + ConfigurationSettings.AppSettings["commentEnd"]);
  yybegin(YYINITIAL);
  return null;
  }


<YYINITIAL>\" {
  Console.Write (ConfigurationSettings.AppSettings["stringStart"] + yytext());
  yybegin(STRING_STATE);
  return null;
  }

<STRING_STATE>\\\" {
  Console.Write (yytext());
  return null;
  }

<STRING_STATE> {LIGNEVIDE} { 
Console.Write(ConfigurationSettings.AppSettings["return"]);
return null;
}

<STRING_STATE> {NEW_LINE} {
  Console.Write(ConfigurationSettings.AppSettings["return"]);
  return null;
  }

<STRING_STATE>\" {
  Console.Write (yytext() + ConfigurationSettings.AppSettings["stringEnd"]);
  yybegin(YYINITIAL);
  return null;
  }
<STRING_STATE> . {
  Console.Write (yytext());
  return null;
  }

<YYINITIAL> {MOTS_CLES} {
  Console.Write (ConfigurationSettings.AppSettings["motCleStart"] + yytext() + ConfigurationSettings.AppSettings["motCleEnd"]);
  return null;
  }

<YYINITIAL> {IDF} {
  Console.Write (yytext());
  return null;
  }

<YYINITIAL> {CHAR} {
  Console.Write (ConfigurationSettings.AppSettings["charStart"] + yytext() + ConfigurationSettings.AppSettings["charEnd"]);
  return null;
  }

<YYINITIAL> {NUMBER} {
  Console.Write (ConfigurationSettings.AppSettings["numberStart"] + yytext() + ConfigurationSettings.AppSettings["numberEnd"]);
  return null;
  }

<YYINITIAL> . {
  Console.Write (yytext());
  return null;
  }
